/* The SREhttp/2 content management system (CMS) program 

   This module will display and create .POL files (poll-definition files)


NOTES on poll implementation:

1) When you use CMS_DEF to define a new dataseries, you can specify
   several poll variables. Actually, up to 5, since poll variables MUST
   be sortable
 
  In step 2 of cms_def you must specify a poll-definition file for all of
  the poll variables you've decided to create.

  Poll definition files have a .POL extension, can be stored in two locations
    a) the "general" directory: CMS\LIB\POL -- these are the common "poll-definitions"
       that can be used by all the dataseries you've defined
    b) the "series specific directory". This will be the "output directory" you
       selected for this dataseries. If you didn't bother selecting an output directory,
       the default will be used (CMS\FILES)

  CMS_DEF will find all .POL files in these two directories, and display a selection
  list. The client will select one of these 
                OR 
        The client will choose the "observation specific poll files"
  
  Observation specific poll-definition files are "specific" to the observation. 
  In other words, in the ELECTIONS dataseries you could have a FAVORITE variable; 
  and for each record (say, each state in the US) you could have a different poll
  (say, each state's candidates would have their own poll-definition file).

  CMS_DEF uses the Poll: line (in the .DEF) file to specify what the poll-definition
  file is for a variable.  
     Poll: varname = a_pol_file ; varname2 = a_pol_file ;
  The a_pol_file should be one of
    i) a file name (with no extension and no path)
       This points to a file in the "general" directory
   ii) a ~ followed by a filename (with no extension and no path)
       This points to a file in the "series specific" directory
  iii) *
      This means "using observation-specific poll-definition files"

2) CMS_ADD is used to add a new record; either from the CMS main screen,
   or using a link with a add-record (and add-record-response) files
   specified.

   In either case, poll files (if specified) will cause one of two
   selection lists to be generated:
    i) for non-observation-specific poll-definition files (cases i and ii above),
       the current set of choices is listed. The client can chose one of these
       choices. Or, which is probably more sensible, he can leave this as is,
       in which cases the poll starts with 0 responses.
   ii) for observation-specific poll definition files (case iii above), the
       list of available poll files is chosen (kind of like what happens 
       in CMS_MAKE when the client is asked to chose a poll-definition file).
        
   In ii, the client will not be able to add a response (the poll will start
   with 0 responses).

   The structure of the poll variable is:
     Nresp c1 c2 ... cn:sfile,the question:
   where
      nresp      : number of responses to date
      c1...cn    : for n choices, the count (of times this choice was chosen)
      :          :  a required seperator
     sfile       : the response file.  The ~ syntax is used. Note that * can
                   not be used here -- since this specifies the observation specific
                   poll-definition file
    Note that for non-observation specific polls, sfile should be the same as 
    the sfile in the .DEF file. However, this is NOT required -- one could
    change (using CMS_MOD) the poll-definition file for this observation.
    
    In other words, all other CMS actions use the poll definition file
    noted in the actual observation. The poll definition files defined
    in the .DEF file are used as "defaults" by CMS_ADD.



*/

/******** User changeable parameters ***/
/* root directory for the SREhttp/2 CMS addon.
   Leave blank to use default (CMS subdirectory of srehttp/2 addon directory */
cmsdir=''


/******** end of user changeable parameters ***/


parse arg list,servername,verb,tempfile,,
          prog_file,reqnum,verbose,client_ip,privset,,
          uri,host_nickname,idinfo,aiter,attribs,seluse2

crlf='0d0a'x

if verb=" " then do
   say "This SREhttp/2 procedure is not meant to be run in standalone mode "
   exit
end  

addon_dir=sre_value('H2_ADDON_DIR_DEFAULT',,'SRE')
if cmsdir='' then cmsdir=strip(addon_dir,,'\')||'\CMS'
foo=dosisdir(cmsdir)
if foo=0 then return 'String Error:unable to find CMS directory: ('cmsdir

cmsdir_data=cmsdir||'\DATA'
okk=dosisdir(cmsdir_data)
if okk=0 then do
  return 'string unable to find CMS data directory: 'cmsdir_data
end

cmsdir_lib=cmsdir||'\LIB'
okk=dosisdir(cmsdir_lib)
if okk=0 then do
  return 'string unable to find CMS index directory: 'cmsdir_lib
end

cmsdir_pol=cmsdir||'\LIB\POL'
okk=dosisdir(cmsdir_pol)
if okk=0 then do
  return 'string unable to find CMS index directory: 'cmsdir_pol
end


cmsdir_index=cmsdir||'\INDEX'
okk=dosisdir(cmsdir_index)
if okk=0 then do
  return 'string unable to find CMS index directory: 'cmsdir_index
end

parse var uri thisaddon '?' .
jj=lastpos('/',thisaddon)
mainaddon=left(uri,jj)||'cms.cmd'

optname.=''
m=0
if list='' then list='action=1'
do until list=""
    parse var list opt1 '&' list
    m=m+1
    parse var opt1 avar '=' aval ; avar=strip(translate(avar)) 
    optname.0=optname.0||' '||avar
    aval=translate(aval,' ', '+')
    aval=sre_packur(aval)
    avar1='!!'avar
    if optname.avar1<>'' then
       optname.avar1=optname.avar1||' '||aval
    else
       optname.avar1=aval
    avar2='!'avar
    aval2=strip(translate(aval,' ','090a0d'x))
    optname.avar2=aval2
end

/***** End of GENERIC CMS addon code *****/


if optname.!action='1' then do   /* choose a record */
  call build_polfile
  r1=result
  return r1
end

if optname.!action='2' then do   /* choose a record */
  call show_polfile
  r1=result
  return r1
end

if optname.!action='4' then do   
  call write_polfile
  r1=result
  return r1
end

if optname.!action='5' then do   
  call show_polfiles
  r1=result
  return r1
end

if optname.!action='21' then do   
  call do_quest1
  r1=result
  return r1
end

if optname.!action='22'  | optname.!_ACTION='22' then do   
  call do_quest2
  r1=result
  return r1
end

if optname.!action='31' then do   
  call do_update
  r1=result
  return r1
end

if strip(translate(optname.!action))='GIF' then do   
  call do_gif
  return r1
end

return 'String Unknown CMS_POLL action: 'optname.!action


/*******************/
/* build a poll file */
build_polfile:


call lineout tempfile,'<html><head><title>CMS: Build a poll-definitions file </title> </head>'

fdir3='' ; fdir2=''
if optname.!FDIR<>'' then do
  fdir2=translate(optname.!fdir,'\','/')
  fdir3='&fdir='||optname.!fdir
end

call lineout tempfile,'<font size=-1>'
if optname.!DOCLOSE=1 then do
  call lineout tempfile,'<a href="javascript:close()">close window</a> ||'
  call lineout tempfile,'<a href="#notes">Description</a> || '
  call lineout tempfile,'<a href="'||thisaddon||'?action=5&doclose=1'||fdir3||'">View a poll-definition file</a> ||'
  call lineout tempfile,'<a href="'||mainaddon||'?action=1">Another CMS action</a>'
end
else do
  call lineout tempfile,'<a href="javascript:close()">close window</a> ||'
  call lineout tempfile,'<a href="'||thisaddon||'?action=5'||fdir3||'">View a poll-definition file</a> ||'
  call lineout tempfile,'<a href="#notes">Description</a> ||'
  call lineout tempfile,'<a href="'||mainaddon||'?action=1">Another CMS action</a>'
end
call lineout tempfile,'</font><br>'
call lineout tempfile,'<h3>Build a poll-definition file</h3><em>for use with <tt>Poll</tt> variables </em><p>'

call lineout tempfile,' You can use this form to create a poll-definition file containing up to 15 possible answers.'
call lineout tempfile,' <br>If you need a longer list of answers, you''ll have to create a text-file'
call lineout tempfile,'using your favorite edtior<br>(see <a href="#notes">below</a> for some details).'
call lineout tempfile,' '
call lineout tempfile,'<form method="POST" name="build" action="'||thisaddon||'">'
call lineout tempfile,'<input type="hidden" name="action" value=4><br> '

call lineout tempfile,'<b>Name of file</b> (no path, no extension): '
call lineout tempfile,'<input type="text" name="OPTNAME" size=15><br> '
call lineout tempfile,'Description: <input type="text" name="desc" size=60><p>'

call lineout tempfile,'<table>'
call lineout tempfile,'<tr bgcolor="#ababab"><th>The question</th>'
call lineout tempfile,'<td><textarea name="QUESTION" rows=3 cols=60>What is the question?</textarea></td>'
call lineout tempfile,'<tr><td colspan=2><em>up to 15 answers ... </em></td>'
call lineout tempfile,'<tr><td colspan=2><table align="center" border=3>'
call lineout tempfile,'<tr bgcolor="#abbb99"><td>&nbsp;</td><th>Value </th><th>displayed</th> '
do mm=1 to 15
   call lineout tempfile,'<tr><td><font size=-1><tt>'||mm||'</tt></font></td>'
   call lineout tempfile,'<td><input type="text" name="OPT_'||mm||'" size=7></td> '
   call lineout tempfile,'<td><input type="text" name="DISP_'||mm||'" size=70></td> '
end
call lineout tempfile,'</td></table>'
call lineout tempfile,'<tr><th colspan=2> &nbsp; </th>'
call lineout tempfile,'<tr bgcolor="#aaaa00"><th>Header (optional)</th>'
call lineout tempfile,'<td><textarea name="HEADER" rows=3 cols=60></textarea></td>'

call lineout tempfile,'<tr  bgcolor="#aaaa00"><th>Footer (optional)</th>'
call lineout tempfile,'<td><textarea name="FOOTER" rows=3 cols=60></textarea></td>'

call lineout tempfile,'<tr  bgcolor="#00aaaa"><th>Survey output file (optional)</th>'
call lineout tempfile,'<td><font size=-1><tt>File name only, do <b>not</b> include path information</tt></font><br>'
call lineout tempfile,'<input type="text" name="SOUTPUT"  size=30><br>'
call lineout tempfile,'<tr  bgcolor="#00aaaa"><th>Survey-output format:</th>'
call lineout tempfile,'<td><textarea name="SFORMAT" rows=3 cols=60></textarea></td>'
call lineout tempfile,'</td>'


call lineout tempfile,'<tr bgcolor="#49ff95" ><th>Color of <br>questionaire box: </th>'
call lineout tempfile,'<td><input type="text" name="COLOR"  size=20></td>'


call lineout tempfile,'</table><p>'


call lineout tempfile,'<input type="submit" value="Create the poll-definition file"><input type="reset"><br> '
call lineout tempfile,' '
call lineout tempfile,' '
call lineout tempfile,' '
call lineout tempfile,' '
call lineout tempfile,'</form> '

call lineout tempfile,'<a name="notes"><h3>Description</h3></a> '
call lineout tempfile,'CMS uses <tt>poll-definition files</tt> to build the answer, list of response, and other '
call lineout tempfile,' components of a questionaire box for <tt>Poll</tt> variables.'
call lineout tempfile,'<ul><li> These poll-definition files are simple text files.'
call lineout tempfile,'<li> They all have a <tt>.POL</tt> extension,'
call lineout tempfile,'<li> Poll-definition files created with this form are stored in the  <em>default</em> <u>poll-definition file storage directory</u> (<em>'||cmsdir_pol||')</em>.<br>'
call lineout tempfile,'    <em>Additionally,</em> CMS will check the CMS dataseries-specific <tt>output directory</tt> for .POL files.'
call lineout tempfile,'  (<tt>'||fdir2||'</tt>)'
call lineout tempfile,'</ul>'
call lineout tempfile,''
call lineout tempfile,'The structure of these .POL files is "mime-like"<pre>keyword: value </pre>:'
call lineout tempfile,'The following keywords are supported:'
call lineout tempfile,'<menu>'
call lineout tempfile,'<li><tt>? </tt>: The text of the question. You can include HTML elements.'
call lineout tempfile,'<li><tt>nn </tt>: where <tt>nn</tt> is any number (integer or real).' 
call lineout tempfile,'<br>The stuff before the : is the internal answer (it is used by CMS)'
call lineout tempfile,'<br>The stuff after the : is the answer (it will be displayed)'
call lineout tempfile,'<br>Since CMS is not particularly sophisticated in how it uses these internal values, '
call lineout tempfile,'we highly recommend using consecutive integers (such as <Tt>1,2,3,...</tt>).'
call lineout tempfile,'<li> <tt>Header</tt> : A header, that is displayed before the question. HTML is okay.'
call lineout tempfile,'<li> <tt>Footer</tt> : A footer, that is displayed after the answers. HTML is okay.'
call lineout tempfile,'<li> <tt>Color</tt> : Background color to use for the questionaire box.'
call lineout tempfile,'<li><tt>SOutput</tt>: A output file to which each response will be written.'
call lineout tempfile,'<br>This file will be in the <tt>POLLS</tt> subdirectory of your series-specfic output-directory.'
call lineout tempfile,'Each line in this file will correspond to one response, and can contain the choice (to the poll),'
call lineout tempfile,'along with responses to additional survey questions.'
call lineout tempfile,'<li><tt>SFormat</tt>: A format line that specifies what, and how, to write results to the survey-output file.'
call lineout tempfile,'<br>This is ignored if <tt>SOutput</tt> is left blank.'
call lineout tempfile,'</menu>'
call lineout tempfile,'<b>Notes:</b> '
call lineout tempfile,'<ul><Li>For readability purposes, you can have multiple  <tt>?, nn, Header,</tt> and <tt>Footer</tt>  lines -- they will be concatenated appopriately.'
call lineout tempfile,'<li>Lines that begin with a semi-colon (;) are comments, and are ignored.'
call lineout tempfile,'<li>For details on survey-forms, see the <a href="/cms/cms.cmd?action=2#appendix_survey">CMS documentation</a>'
call lineout tempfile,'</ul>'
call lineout tempfile,'<table bgcolor="#aabbaa"><tr><th>Example&nbsp;&nbsp;</th><td><Pre>'
call lineout tempfile,'; this is an example of a poll definition file'
call lineout tempfile,''
call lineout tempfile,'Header: &lt;em&gt;we are interested in your opinions....&lt;/em&gt;&lt;p&gt;'
call lineout tempfile,''
call lineout tempfile,'?: How would you rate this comment?'
call lineout tempfile,''
call lineout tempfile,'1: Uninteresting'
call lineout tempfile,'2: Mildly interesting'
call lineout tempfile,'3: Interesting'
call lineout tempfile,'4: Uninteresting'
call lineout tempfile,''
call lineout tempfile,'Footer:  &lt;p&gt;&lt;em&gt;Thanks for your input &lt;/em&gt;'
call lineout tempfile,'Footer:  &lt;br&gt;Come back again, soon!'
call lineout tempfile,''
call lineout tempfile,'SOutput: survey1.out' 
call lineout tempfile,'SFormat: _ID _VAR , _POLL :: INCOME     AGE     SEX '

call lineout tempfile,''
call lineout tempfile,'color: cyan'



call lineout tempfile,'</pre></td></table> '

gooble:         /* jump here on error */

call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode


/*******************/
/* write to  a stored file */
write_polfile:


call lineout tempfile,'<html><head><title>CMS: Write a poll-definition file </title> </head>'
call lineout tempfile,'<font size=-1><a href="'||mainaddon||'?action=1">Another CMS action</a></font><br>'

call lineout tempfile,'<body><h3>Creating a Poll-definition File</h3>'
daindx=optname.!optname
ff=cmsdir_pol||'\'||daindx||'.POL'
goo=stream(ff,'c','query exists')
if goo<>'' then do
   call lineout tempfile,'<b>Error</b>: file already exists: '||ff
   signal gooble
end

aa=';Created '||date('n')||' '||time('n')||'0d0a'x
aa=aa||'; DESC = '||optname.!DESC||'0d0a'x
aa=aa||'0d0a'x
if optname.!HEADER<>"" then aa=aa||'HEADER: '||optname.!HEADER||'0d0a'x

aa=aa||'?: '||optname.!QUESTION||'0d0a'x

do mm=1 to 15
   vv='!OPT_'||mm
   if optname.vv='' then iterate
   vv2='!DISP_'||mm
   aa=aa||optname.vv||' : '||optname.vv2||'0d0a'x
end 

if optname.!FOOTER<>"" then aa=aa||'FOOTER: '||optname.!FOOTER||'0d0a'x

if optname.!COLOR<>"" then aa=aa||'COLOR: '||optname.!COLOR||'0d0a'x
if optname.!SOUTPUT<>"" then aa=aa||'SOUTPUT: '||optname.!SOUTPUT||'0d0a'x
if optname.!SFORMAT<>"" then aa=aa||'SFORMAT: '||optname.!SFORMAT||'0d0a'x

arf=sre_append_file(ff,aa,2,,1)
if arf<0 then do
   call lineout tempfile,'<b>Error</b>: unable to write .POL file 'ff
   signal gooble
end 
call lineout tempfile,'<br> File succesfully created:<tt> 'ff||'</tt><p>'
call lineout tempfile,'It can be used with <tt>Poll</tt> variables -- choose <u>'daindx'</u>'
call lineout tempfile,'<p>Would you like to <a href="'||thisaddon||'?action=5">view the contents</a> of a poll-definition file?'
gooble:
call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode


/*******************/
/* choose a stored file to view*/
show_polfiles:

call sec_check
res1=result
if res1<>0 then return res1


call lineout tempfile,'<html><head><title>CMS: Choose a poll-definition file to view </title> '

call lineout tempfile,'<script language="javascript">'

call lineout tempfile,'function show_opts(isblur,afile){'
call lineout tempfile,'dahelp=window.open("/cms/cms_poll?action=2&file="+afile,"HELPER",' ||,
        '"location=NO,MENUBAR=NO,RESIZABLE,DEPENDENT,SCROLLBARS=YES,STATUS=YES,TITLEBAR,TOOLBAR=NO,SCREENX=12,SCREENY=30,height=350,width=350");' 
call lineout tempfile,''
call lineout tempfile,'if (isblur==1) ; {dahelp.blur() }'
call lineout tempfile,' }'
call lineout tempfile,' '
call lineout tempfile,'</script>'
call lineout tempfile,'</head>'
call lineout tempfile,'<font size=-1>'
if optname.!DOCLOSE=1 then do
   call lineout tempfile,'<a href="javascript:close()">close window</a> ||  '
   call lineout tempfile,'<a href="'||mainaddon||'?action=1">Another CMS action</a> || '
   call lineout tempfile,'<a href="'||thisaddon||'?action=1&doclose=1">Create a poll-definition file</a>  '
end
else do
   call lineout tempfile,'<a href="'||mainaddon||'?action=1">Another CMS action</a> || '
   call lineout tempfile,'<a href="'||thisaddon||'?action=1">Create a poll-definition file</a>  '
end
call lineout tempfile,' </font><br>'

call lineout tempfile,'<h3>CMS: Choose a poll-definition file to view </h3> '

oo=sysfiletree(cmsdir_pol||'\*.POL','ofiles','FO')
if ofiles.0=0 then do
   call lineout tempfile,   aa='There are <b>no</b> poll-definition files (.POL files) currently available in '||cmsdir_pol||'</b>'
   signal gooble
end
ofiles.!0=ofiles.0

fdir=''
ofiles2.0=0
if  optname.!FDIR<>'' then do
   fdir=translate(optname.!fdir,'\','/')
   oo2=sysfiletree(fdir||'\POLLS\*.POL','ofiles2','FO')
   i00=ofiles.0
   do iarf=1 to ofiles2.0
       fnew=ofiles2.iarf
       i00=i00+1
       ofiles.i00=fnew
   end 
   ofiles.0=i00
end

if (ofiles.0)=0 then do
   call lineout tempfile, 'There are <b>no</b> poll-definition files (.POL files) currently available in '||cmsdir_pol||'</b>'
   if fdir<>'' then   call lineout tempfile,' <em>or</em> in <tt>'||fdir||'</tt><br>'
   signal gooble
end

call lineout tempfile,'<form target="HELPER" name="viewopt" action="'||thisaddon||'" method="POST" '
call lineout tempfile,'onSubmit="show_opts(0,document.viewopt.file.value); ">'

call lineout tempfile,'<input type="hidden" name="action" value=2>'
if optname.!fdir<>'' then call lineout tempfile,'<input type="hidden" name="fdir" value="'||optname.!Fdir||'">'

call lineout tempfile,'Select one of these '||ofiles.0||' to view, then hit the <input type="submit" value="View file"> button.<p>'


do j1=1 to ofiles.0
    ofiles_name.j1=translate(filespec('n',ofiles.j1))
    parse var ofiles_name.j1 ofiles_name.j1 '.' .
end 


call lineout tempfile,'<p><table>'
bg.1='bgcolor="#ddeedd"'
bg.2='bgcolor="#ddddcc"'
do jj=1 to ofiles.0

  if jj=1 & ofiles.0<>ofiles.!0 then do
     call lineout tempfile,'<tr bgcolor="#d1d1d1"><td colspan=2><em>generic .POL files </em></td>'
  end /* do */

  if jj=ofiles.!0+1 then do
     call lineout tempfile,'<tr bgcolor="#d1d1d1"><td colspan=2><em>data series specfic .POL files </em></td>'
  end /* do */

  ijj=jj//2 + 1
  aa=sre_read_file(ofiles.jj,2,2,1,400)   
  adesc=''
  do until aa=''
    parse var aa aline '0d0a'x aa
    aline=strip(aline); 
    if abbrev(aline,';')=0 then iterate
    parse var aline . ';' aword '=' aline
    if strip(translate(aword))='DESC' then do
      adesc=aline
      leave
    end
  end 
  atwiddle=''
  if jj>ofiles.!0 then atwiddle='~'
  call lineout tempfile,'<tr '||bg.ijj||'><td> <input type="radio" name="file" value="'||atwiddle||ofiles_name.jj||'">'||ofiles_name.jj||'</td>'
  call lineout tempfile,'<td>'
  if adesc<>'' then call lineout tempfile,'  <em>'||adesc||'</em>'
  call lineout tempfile,'</td>'


end

call lineout tempfile,'</table><br>'
call lineout tempfile,'<input type="submit" value="View file">.'

call lineout tempfile,'</form>'

call lineout tempfile,'<hr><b>The following directories are checked for .POL files</b>'
call lineout tempfile,'<menu><li>The <em>default</em> poll-definition file directory: <tt>'||cmsdir_pol||'</tt> '
if fdir<>'' then call lineout tempfile,'<li>The <em>series-speciificdefault</em> poll-definition file directory: <tt>'||fdir||'</tt> '
call lineout tempfile,'</menu>'

gooble:         /* jump here on error */

call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode

/*******************/
/* view a stored file */
show_polfile:

call sec_check
res1=result
if res1<>0 then return res1


 call lineout tempfile,'<html><head><title>CMS: View a poll-definition file </title></head> '

daindx=strip(optname.!file)
fdir=strip(translate(optname.!fdir,'\','/'))

if daindx='*' then do
 call lineout tempfile,'<font size=-1><a href="javascript:close()">close window</a></font><p>'
 call lineout tempfile,'<b>You''ve selected </b><tt>observation specific polls</tt>.'    
 call lineout tempfile,'This means that, for this variable, each observation will be assigned '
 call lineout tempfile,'a unique poll-definition file (though multiple observations can share a single poll-definition file).'
 call lineout tempfile,'<p>When you create a new record, you''ll be asked to specify a poll-definition file.'
 signal gooble
end /* do */

if daindx='-' then do
 call lineout tempfile,'<font size=-1><a href="javascript:close()">close window</a></font><p>'
 call lineout tempfile,'<b>You need to select a </b><tt>poll-definition</tt> file.<p>'    
 call lineout tempfile,'<em>For each poll variable, you must specify a poll-definition file</em>.<br>'
 signal gooble
end /* do */


if abbrev(daindx,'~')=1 then do
    daindx=substr(daindx,2)
    usedir=fdir||'\POLLS'
end
else do
   usedir=cmsdir_pol
end 

ff=usedir||'\'||daindx||'.POL'
goo=stream(ff,'c','query exists')
if goo='' then do
   call lineout tempfile,'Error: could not find a CMS poll-definition: '||ff
   signal gooble
end

stuff=sre_file_read(ff,2,2,1)
if stuff='' then do
   call lineout tempfile,'Error: empty a CMS poll-definition file: '||ff
   signal gooble
end

adesc=''
ith=0
vopts.=''
do until stuff=''
   parse var stuff aline '0d0a'x stuff
   aline=strip(aline)
   if aline='' then iterate
   if abbrev(aline,';')=1 then do
      parse var aline . ';' aword '=' aline
      if strip(translate(aword))='DESC' then adesc=aline
      iterate
   end

   parse var aline a1 ':' a2
   a1=strip(translate(a1))
   r1=a1
   if a1='?' then r1="The question"
   if datatype(a1)='NUM' then r1='Answer: <tt>'||a1||'</tt>'
   va1='!'||a1
   if vopts.va1='' then do
        vopts.va1=strip(a2)
        vopts.va1.1=r1
        vopts.0=vopts.0||' '||a1
   end
   else do
        vopts.va1=vopts.va1||'0d0a'x||strip(a2)
   end
end   
call lineout tempfile,'<font size=-1><a href="javascript:close()">close window</a></font><p>'
call lineout tempfile,'<u>Description</u>: <b>'||adesc||'</b><p>'

call lineout tempfile,'<table>'
   call lineout tempfile,'<tr><th>Value</th><th>Display</th>'

do jj=1 to words(vopts.0)
   a1=translate(strip(word(vopts.0,jj)))
   va1='!'||a1
   call lineout tempfile,'<tr><td><tt>'||vopts.va1.1||'</tt>'
   select 
      when a1="COLOR" then do
         call lineout tempfile,'<td><tt>'||vopts.va1||'</tt>'
         call lineout tempfile,' (<font color="'||vopts.va1||'">'||vopts.va1||'</font>)</td>'
      end
      when wordpos(a1,'?')>0 then do
         call lineout tempfile,'<td><b>'||vopts.va1||'<b></td>'
      end
      when wordpos(a1,'SOUTPUT SFORMAT COLOR HEADER FOOTER')>0 then do
         call lineout tempfile,'<td><tt>'||vopts.va1||'<tt></td>'
      end
      otherwise do
         call lineout tempfile,'<td><em>'||vopts.va1||'</em></td>'
      end
   end
end
call lineout tempfile,'</table>'


call lineout tempfile,'<p><b>Note:</b> this poll definition file is: 'ff
gooble:         /* jump here on error */

call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode


/************************/
/* build a questionaire box, using poll file associated with a poll file question */
do_quest1:

call lineout tempfile,'<html><head><title>CMS: A Poll questionaire </title></head>'


oy=''
if optname.!bgcolor<>"" then do
   oy=' bgcolor="'||optname.!bgcolor||'" '
end 
call lineout tempfile,'<body '||oy||' >'


/* read .def file (mostly for .!filedir ) */
daindx=optname.!deffile
ff=cmsdir_index||'\'||daindx
goo=stream(ff,'c','query exists')
if goo='' then do
   call lineout tempfile
   return 'String CMS_POLL error: could not find index file: '||ff
end
call read_deffile ff

pfile0=strip(optname.!pfile)

if abbrev(pfile0,'~')=1 then do
   usedir=stuff.!filedir
   pfile=substr(pfile0,2)||'\POLLS'
end 
else do
   pfile=pfile0
   usedir=cmsdir_pol
end 
  
ff=usedir||'\'||pfile||'.POL'
goo=stream(ff,'c','query exists')
if goo='' then do
   call lineout tempfile
    return 'string <b>CMS_POLL Error:</b> could not find a CMS poll-definition: '||ff
end

polstuff=sre_read_file(ff,2,2,1)
if polstuff='' then do
   call lineout tempfile
    return 'string <b>CMS_POLL Error:</b> Empty poll-definition file '||ff
end

vopts.=''
nopts=0 ;noptlist=''
do until polstuff=''
    parse var polstuff aline '0d0a'x polstuff
    aline=strip(aline)
    if aline='' then iterate
    if abbrev(aline,';')=1 then iterate
    parse var aline a1 ':' a2
    a1=strip(translate(a1))
    if datatype(a1)='NUM' then do
       if wordpos(a1,noptlist)=0 then do
         nopts=nopts+1
         vopts.nopts=a1
         vopts.nopts.2=a2
         noptlist=noptlist||' '||a1
       end
       else do
         vopts.a1.2=vopts.a1.2||'0d0a'x||a2
       end
    end 
    va1='!'||a1
    vopts.va1=strip(vopts.va1||' '||a2)
end   

formstuff='<!--_POLL-->'
if optname.!FORM<>'' then do   /* use a survey form file */
  ffile=stuff.!filedir||'\POLLS\'||filespec('n',strip(optname.!form))
  formstuff=sre_read_file(ffile,2,2,1)   
  if formstuff='' then do
     formstuff="<b>Warning:</b> empty FORM file: "ffile
  end 
end 

if optname.!TARGET<>'' then
  call lineout tempfile,'<font size=-1><a href="javascript:close()">close window</a></font><br>'

if optname.!H3<>'' then 
   call lineout tempfile,'<h3>'|| sre_packur(translate(optname.!H3,' ','+'))||'</h3>'

if optname.!DESC<>'' then 
   call lineout tempfile,'<br><b>'|| sre_packur(translate(optname.!desc,' ','+'))||'</b>'

bgcolor=''
if vopts.!COLOR<>'' then bgcolor=' bgcolor="'||vopts.!color||'" '
call lineout tempfile,'<form name="quest1" method="POST" action="'||thisaddon||'">'
call lineout tempfile,'<input type="hidden" name="_action" value=22>'

daindx=optname.!deffile
call lineout tempfile,'<input type="hidden" name="_deffile" value="'||daindx||'">'

svarname=optname.!VAR
call lineout tempfile,'<input type="hidden" name="_var" value="'||svarname||'">'

call lineout tempfile,'<input type="hidden" name="_sfile" value="'||pfile0||'">'

if optname.!TARGET<>'' then
  call lineout tempfile,'<input type="hidden" name="_target" value="1">'

id7=optname.!ID
call lineout tempfile,'<input type="hidden" name="_id" value="'||id7||'">'

dt=translate(time('n')||' '||date('n'),'+',' ')
call lineout tempfile,'<input type="hidden" name="_datetime" value="'||dt||'">'


call lineout tempfile,'<tr><td>'


tt='!?'
qq='<table border=1 '||bgcolor||' >'||'0d0a'x
if vopts.!HEADER<>'' then 
  qq=qq||'<tr><td>'||vopts.!HEADER||'</td>'||'0d0a'x
qq=qq||'<tr><td><b>'||vopts.tt||'</b> '||'0d0a'x
qq=qq||'<ol type="i">'||'0d0a'x
do jj=1 to nopts
  qq=qq||'<li> <input type="radio" name="_poll" value="'||vopts.jj||'">'||'0d0a'x
  qq=qq||'<em>'||vopts.jj.2||'</em>'||'0d0a'x
end
qq=qq||'</ol></td>'||'0d0a'x
if vopts.!footer<>'' then 
  qq=qq||'<tr><td>'||vopts.!FOOTER||'</td>'||'0d0a'x
qq=qq||'</table>'||'0d0a'x

ff=sre_replacestrg(formstuff,'<!--_POLL-->',qq,'ALL')

call lineout tempfile,ff


call lineout tempfile,'<input type="submit" value="Submit your choice">'
call lineout tempfile,'</form>'
gooble:         /* jump here on error */

call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode

/************************/
/* record an answer to a questionaire (poll) */
do_quest2:

call lineout tempfile,'<html><head><title>CMS: saving poll response </title></head>'

oy=''
if optname.!bgcolor<>"" then do
   oy=' bgcolor="'||optname.!bgcolor||'" '
end 
call lineout tempfile,'<body '||oy||' >'

tt=optname.!_TARGET
if tt='' then tt=optname.!TARGET
if tt<>'' then
 call lineout tempfile,'<font size=-1><a href="javascript:close()">close window</a></font><br>'


daindx=optname.!_DEFFILE
if daindx='' then daindx=optname.!DEFFILE

ff=cmsdir_index||'\'||daindx
goo=stream(ff,'c','query exists')
if goo='' then do
   call lineout tempfile
   return 'String CMS_POLL error: could not find index file: '||ff
end

call read_deffile ff

rcode=check_access(stuff.!OWNERS,stuff.!SERIES)
if rcode<>1 then return rcode

a=sre_cache_data(stuff.!series,'OPEN  INDEX',stuff.!file)
if a>0 then do
   amess=sre_cache_data('OPEN','ERROR',a,stuff.!file)
   call lineout tempfile
   return 'String CMS_POLL error 1:  '||amess
end

svarname=optname.!_VAR
if svarname='' then svarname=optname.!VAR

findid=optname.!_ID
if findid='' then svarname=optname.!ID

astat=sre_cache_data(stuff.!series,'FIND ID7',findid,svarname)

parse var astat astat0 astat
if astat0<>0 then do
      a=sre_cache_data(stuff.!series,'CLOSE') 
      amess=sre_cache_data('FIND','ERROR',astat0,svarname)
      call lineout tempfile
      a=sre_cache_data(stuff.!series,'CLOSE') 
      return 'String CMS_POLL error 2: 'findid ','amess
end 

parse var astat . '0d0a'x  vval

parse var vval vval1 ':' poll_misc
parse var vval1 totresponses counts 
if totresponses='' then totresponses=0

/* now open up basic poll file info  */


pfile0=strip(optname.!_SFILE)
if pfile0='' then pfile0=strip(optname.!SFILE)

if abbrev(pfile0,'~')=1 then do
   usedir=stuff.!filedir
   pfile=substr(pfile0,2)||'\POLLS'
end 
else do
   pfile=pfile0
   usedir=cmsdir_pol
end 
ff=usedir||'\'||pfile||'.POL'

goo=stream(ff,'c','query exists')
if goo='' then do
    call lineout tempfile
    a=sre_cache_data(stuff.!series,'CLOSE') 
    return 'string <b>CMS_POLL Error:</b> could not find a CMS poll-definition: '||ff
end

polstuff=sre_read_file(ff,2,2,1)
if polstuff='' then do
    call lineout tempfile
    a=sre_cache_data(stuff.!series,'CLOSE') 
    return 'string <b>CMS_POLL Error:</b> Empty poll-definition file '||ff
end

vopts.=''
nopts=0 ;noptlist=''
do until polstuff=''
    parse var polstuff aline '0d0a'x polstuff
    aline=strip(aline)
    if aline='' then iterate
    if abbrev(aline,';')=1 then iterate
    parse var aline a1 ':' a2
    a1=strip(translate(a1))
    va1='!'||a1 
    if datatype(a1)='NUM' then do
      if wordpos(a1,noptlist)=0 then do
         nopts=nopts+1
         vopts.nopts=a1
         vopts.nopts.2=a2
         noptlist=noptlist||' '||a1
         tt='!'||a1
         vopts.tt.1=nopts
       end
       else do
         vopts.a1.2=vopts.a1.2||'0d0a'x||a2
       end
    end 
    va1='!'||a1
    vopts.va1=strip(vopts.va1||' '||a2)

end   

cc=strip(optname.!_POLL)  /* the "stored" value */

if cc="" then do 
     a=sre_cache_data(stuff.!series,'CLOSE')    /* nothing will be updated */ 
     signal no_poll   /* skip to survey section */
end

vcc='!'||cc
ndo=strip(vopts.vcc.1)   /* the index to this stored value (in the counts vector */
if datatype(ndo)<>'NUM' then do
    call lineout tempfile
    a=sre_cache_data(stuff.!series,'CLOSE') 
    return 'string  <b>CMS_POLL Error:</b> a bad choice indicator: 'cc','ndo','vcc
end

do jj=1 to nopts
   dvalue.jj=strip(word(counts,jj))
   if dvalue.jj='' then dvalue.jj=0
   if datatype(dvalue.jj)<>'NUM' then do
     call lineout tempfile
     a=sre_cache_data(stuff.!series,'CLOSE') 
     return 'string <b>CMS_POLL Error:</b> bad count: 'counts
   end
end 

dvalue.ndo=dvalue.ndo+1
totresponses=totresponses+1

newstring=totresponses||' '
do jj=1 to nopts
   newstring=newstring||dvalue.jj||' '
end
if length(vval1)> length(newstring) then newstring=left(newstring,length(vval1),' ')

tosave=newstring||':'||poll_misc


aa=sre_cache_data(stuff.!series,'MODIFY ID7 VAR',findid,svarname,tosave)
if length(poo)=1 then do
   amess=sre_cache_data('MODIFY','ERROR',poo)
   call lineout tempfile,'<b>Error:</b> '||amess
   a=sre_cache_data(stuff.!series,'CLOSE') 
   signal gooble
end
a=sre_cache_data(stuff.!series,'CLOSE') 


call lineout tempfile,'<h3>Your response has been saved</h3>'

Call lineout tempfile,'<dl>'
Call lineout tempfile,'<dt>For the poll:'
tt='!?'
Call lineout tempfile,'<dd><em>'||vopts.tt||'</em>'
Call lineout tempfile,'<dt>your response:'
Call lineout tempfile,'<dl><tt>'||vopts.ndo.2||'</tt>'
Call lineout tempfile,'</dl>'

Call lineout tempfile,'has been saved.'


/* write to survey output file? */
no_poll: ;
if vopts.!SOUTPUT<>'' & vopts.!SFORMAT<>"" then do     /* output file, with format line, specified? */
   ofile=stuff.!filedir||'\POLLS\'||strip(vopts.!SOUTPUT)
   oline=vopts.!SFORMAT ; uformat=translate(oline)
   do jj=1 to words(optname.0)

       aw=strip(translate(word(optname.0,jj)))
       if pos(aw,uformat)=0 then iterate
       vaw='!'||aw 
       aval=sre_packur(translate(optname.vaw,' ','+'))
       oline=sre_replacestrg(oline,aw,aval,'ALL')
   end
/* now check for ~nn; */
   newoline=''
   do forever 
        if pos('~',oline)=0 then do
           newoline=newoline||oline
           leave
        end 
        parse var oline p1 '~' p2
        if pos(';',p2)=0 then do
           newoline=newoline||oline
           leave
        end 
        parse var p2 ann ';' oline
        if datatype(translate(ann,'x',' '))<>'NUM' then do  /* don't permit spaces in ann */
           newoline=newoline||p1||'~'||ann||';'
           iterate
        end 
        newoline=newoline||p1
        if length(newoline) > (ann-2) then iterate  /* too long already? */
        newoline=left(newoline,ann-1,' ')
   end
   astat=sre_append_file(ofile,newoline,2,1)
   if astat<0 then do
        call lineout tempfile,'<p><b>Warning:</b> Unable to write survey results to output file.'
   end 
   else do
        call lineout tempfile,'<p>Survey results have been saved.'
   end
end


gooble:         /* jump here on error */

call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode


/*************/
/*return a gif file (in the POL directory): action=GIF color=N or color=color_name
 (CMS shipped with RED GREEN BLUE YELLOW MAGENTA CYAN BLACK and GREY color boxes */

do_gif:

ado=optname.!COLOR
if datatype(ado)='NUM' then do  /* semi random draw from gif files */
   aa=sysfiletree(cmsdir_pol||'\*.GIF','fils.','FO')
   if fils.0=0 then return 'String No GIF files available'
   ido=ado//fils.0 ; if ido=0 then ido=fils.0
   rr=sre_command('FILE type image/gif name '||fils.ido,,idinfo)
   return rr
end 
lookfor=cmsdir_pol||'\'||ado||'.GIF'
if stream(lookfor,'c','query exists')='' then do
  lookfor=cmsdir_pol||'\'||ado||_BOX'.GIF'
  if stream(lookfor,'c','query exists')='' then do
       return 'String No Such GIF file: 'ado
  end
end
rr=sre_command('FILE type image/gif name '||lookfor,,idinfo)
return rr



/*************/
/* read a definition file */
read_deffile:
parse arg afile
stuff.=''
stuff=sre_read_file(afile,2,2,1)

do until stuff=''
   parse var stuff aline '0d0a'x stuff ; aline=strip(aline)
   if aline='' then iterate
   if abbrev(aline,';')=1 then iterate
   parse var aline aname ':' avalue
   aname0=strip(translate(aname))
   aname='!'||aname0
   if stuff.aname='' then
      stuff.aname=strip(avalue)
   else
      stuff.aname=stuff.aname||'0d0a'x||strip(avalue)
   stuff.0=stuff.0||' '||aname0
end

return 1




/************************/
/* update results to a questionaire (poll) */
do_update:

call lineout tempfile,'<html><head><title>CMS: updated poll results </title></head>'

oy=''
if optname.!bgcolor<>"" then do
   oy=' bgcolor="'||optname.!bgcolor||'" '
end 
call lineout tempfile,'<body '||oy||' >'


if optname.!TARGET<>'' then
 call lineout tempfile,'<font size=-1><a href="javascript:close()">close window</a></font><br>'


daindx=optname.!deffile
ff=cmsdir_index||'\'||daindx
goo=stream(ff,'c','query exists')
if goo='' then do
   call lineout tempfile
   return 'String CMS_POLL error: could not find index file: '||ff
end

call read_deffile ff

rcode=check_access(stuff.!OWNERS,stuff.!SERIES)
if rcode<>1 then return rcode

a=sre_cache_data(stuff.!series,'OPEN  INDEX',stuff.!file)
if a>0 then do
   amess=sre_cache_data('OPEN','ERROR',a,stuff.!file)
   call lineout tempfile
   return 'String CMS_POLL error 1:  '||amess
end

parse var optname.!chartspec chartonly ',' i250 ',' iheight ',' gcolors
if datatype(i250)<>'NUM' then i250=250
if datatype(iheight)<>'NUM' then iheight=20
if strip(chartonly)=1 | chartonly='' then chartonly='CRS'


svarname=optname.!VAR
findid=optname.!ID
astat=sre_cache_data(stuff.!series,'FIND ID7',findid,svarname)

parse var astat astat0 astat
if astat0<>0 then do
      a=sre_cache_data(stuff.!series,'CLOSE') 
      amess=sre_cache_data('FIND','ERROR',astat0,svarname)
      call lineout tempfile
      return 'String CMS_POLL error 2: 'findid ','amess
end 
a=sre_cache_data(stuff.!series,'CLOSE') 


parse var astat . '0d0a'x  vval

parse var vval vval1 ':' poll_misc
parse var vval1 totresponses counts 
if totresponses='' then totresponses=0

/* now open up basic poll file info  */

pfile0=strip(optname.!pfile)
if abbrev(pfile0,'~')=1 then do
   usedir=stuff.!filedir||'\POLLS'
   pfile=substr(pfile0,2)
end 
else do
   pfile=pfile0
   usedir=cmsdir_pol
end 
ff=usedir||'\'||pfile||'.POL'

goo=stream(ff,'c','query exists')
if goo='' then do
    call lineout tempfile
    a=sre_cache_data(stuff.!series,'CLOSE') 
    return 'string <b>CMS_POLL Error:</b> could not find a CMS poll-definition: '||ff
end


polstuff=sre_read_file(ff,2,2,1)
if polstuff='' then do
   call lineout tempfile
    return 'string <b>CMS_POLL Error:</b> Empty poll-definition file '||ff
end

vopts.=''
nopts=0 ;noptlist=''
do until polstuff=''
    parse var polstuff aline '0d0a'x polstuff
    aline=strip(aline)
    if aline='' then iterate
    if abbrev(aline,';')=1 then iterate
    parse var aline a1 ':' a2
    a1=strip(translate(a1))
    va1='!'||a1
    vopts.va1=a2
    if datatype(a1)='NUM' then do
       if wordpos(a1,noptlist)=0 then do
         nopts=nopts+1
         vopts.nopts=a1
         vopts.nopts.2=a2
         vopts.va1.1=nopts
         noptlist=noptlist||' '||a1
       end
       else do
         vopts.a2.2=vopts.a2.2||'0d0a'x||a2
       end
    end 
end   

/* now compute the stats */
outstuff='<table>'

/* compute the various stats  */  
outstuff=outstuff||'<tr><td align="left" colspan=2>'||'0d0a'x
doall=0 ; if inf_allstats<>'' then doall=1
do jj=1 to nopts
       dacount.jj=strip(word(counts,jj))
       if datatype(dacount.jj)<>'NUM' then dacount.jj=0
       wt.jj=jj
end
       
tsum=0 ; tsum2=0 ; totr=0 ; amax=1
do jj=1 to nopts
        totr=totr+dacount.jj
        if dacount.jj>dacount.amax then amax=jj
        runsum.jj=totr
        t1=(dacount.jj*wt.jj)
        tsum=tsum+t1
        tsum2=tsum2+(t1*t1)
end
if totr=0 then do
  amean=0 ;asd=0;amedian=1;
end
else do
  amean=tsum/totr
  if totr>1 then
     asd=(tsum2-(totr*(amean*amean)))/(totr-1)
  else
     asd=0
  med2=totr/2; amedian=nopts
  do jj=1 to nopts
       if runsum.jj>med2 then do
            amedian=jj
            leave
       end
  end
end

/* got the stats.  output? */

if optname.!h3<>'' then 
   call lineout tempfile,'<h3 align="center">'|| sre_packur(translate(optname.!H3,' ','+'))||'</h3>'

if optname.!DESC<>'' then 
   call lineout tempfile,'<br><b>'|| sre_packur(translate(optname.!desc,' ','+'))||'</b>'

tt='!?'
call lineout tempfile,'<table align="center" width=300 border=1><tr><td align="center"><em>'||vopts.tt||'</em></td></table>'


if pos('S',chartonly)>0 then do
outstuff=outstuff||'<ul>'||'0d0a'x
outstuff=outstuff||'<li>Total # of responses: <tt>'||totr||'</tt>'||'0d0a'x
outstuff=outstuff||'<li> Mean: <tt>'||amean||'</tt>'||'0d0a'x
outstuff=outstuff||'<li> Median: <tt>'||amedian||' (out of '||nopts||')</tt>'||'0d0a'x
outstuff=outstuff||'<li> Max: <tt> choice # '||amax||'</tt>'||'0d0a'x
outstuff=outstuff||'</ul>'||'0d0a'x
end

   ithc=0
   if gcolors<>'' then do
     gcolors.=''
     ithc=0
     do until gcolors=''
        parse var gcolors aco ',' gcolors
        if aco='' then leave
        ithc=ithc+1
        gcolors.ithc=aco
     end 
   end 

if pos('R',chartonly)+pos('C',chartonly)>0 then call g1_it

call lineout tempfile,outstuff

gooble:         /* jump here on error */

call lineout tempfile,'</body> </html>'
call lineout tempfile
rcode=sre_command('FILE ETAG_AUTO EXPIRES 30s NOCACHE NOIF ERASE TYPE text/html NAME 'tempfile,,idinfo)
return rcode



return 'string ok'



/*****/
g1_it:

outstuff=outstuff||'<p><table align="left" border=1 cellpadding=0 cellspacing=0 >'||'0d0a'x
outstuff=outstuff||'<tr>'
outstuff=outstuff||'<td><em>choices</em></td>'
if pos('R',chartonly)>0 then outstuff=outstuff||'<td><tt>count</tt></td>'||'0d0a'x
if pos('C',chartonly) > 0  then  do
  outstuff=outstuff||'<td align="middle"><img align="middle" alt="chart_top" height=2 '||,
             ' width='||i250||' src="/cms/cms_poll.cmd?action=GIF&color=BLACK_BOX&file.gif">'||,
             '</td>'||'0d0a'x
end

cnn=0
do nn=1 to nopts
      cnn=cnn+1
      if ithc>0 then do
         if cnn>ithc then cnn=1
         colordo=gcolors.cnn
      end
      else do
         colordo=cnn
      end 
      outstuff=outstuff||'<tr>'
       outstuff=outstuff||'<td >'||vopts.nn.2||'</td>'
      if pos('R',chartonly)>0 then 
         outstuff=outstuff||'<td>&nbsp;<tt>'||dacount.nn||'</tt></td>'||'0d0a'x
      ilen=0
      if totr>0 then
         ilen=trunc((dacount.nn/totr)*i250)
      if pos('C',chartonly)>0 then  do
        if ilen>0 then do
           outstuff=outstuff||'<td align="middle"><img align="middle" alt="chart_bar" height='||iheight||' '||,
              ' width='||ilen||' src="/cms/cms_poll.cmd?action=GIF&color='||colordo||'&file.gif">'||,
             '</td>'||'0d0a'x
        end
        else do
          outstuff=outstuff||'<td>&nbsp;</td>'||'0d0a'x
        end
      end
end
outstuff=outstuff||'</table><p>'

outstuff=outstuff||'</td>'||'0d0a'x

outstuff=outstuff||'</table>'||'0d0a'x
return 1

/******************/
/* access allowed? */
sec_check:
ss=sre_value('SECURITY_LEVEL',,'SRESYS')
if ss=0 then do
  foo=sre_command('String Sorry. Remote access to configuration utilities not permitted.',,idinfo)
  return foo
end
if ss=1 then do
    myaddr=sre_extract("clientaddr",idinfo)
    moreaddr=sre_value('SERVERADDR_INHOUSE',,'SRESYS')
    who=sre_extract("serveraddr",idinfo)||' '||moreaddr
    if wordpos(myaddr,who)=0 then do
      sfoo=sre_command('String External access to configuration utilities is not permitted' ,,idinfo)
      return sfoo
   end
end

/* else, normal superuser checking */

if wordpos('CMS_DEF',translate(privset))>0 then return 0     
if wordpos('SUPERUSER',translate(privset))>0 then return 0   

/* not enough privs */
if aiter='' then return 'PRIVS Viewing/configuring a CMS dataseries requires superuser privileges '
return 'AUTH Viewing/Defining a CMS dataseries requires that you have superuser privileges '


/******************************/
/* check access rights. Return 1 if okay. Otherwise, return a string to be
  sent back to SREhttp/2 (an authorization response string) */
check_access:procedure expose privset host_nickname idinfo uri aiter
parse  upper arg owners,series

if owners='' then return 1      /* no owners listed, means open access */


tpr=translate(privset)
pneeded=space(translate('CMS:'||strip(series)),0)
if wordpos(pneeded,tpr)>0 then return 1          /* explicit privilege for this */
if wordpos('SUPERUSER',tpr)>0 then return 1

/*  DO not set cookie for comment adding */
/* === look_cook='CMS_'||series
aa=sreh2_get_cookie(lookcook)
if aa=1 | aa  then return 1
====== */

if aiter=''  then do            /* first time, so recheck privs */
  return 'PRIVS   Access to this dataseries is restricted to specific users '
end 

/* otherwise, see if a username has been sent  */
username=''
authh=sre_reqfield('Authorization')
if authh<>'' then username=sre_extract_username(authh) 
if username='' then do             /* no username available */
  return  'AUTH  Access to this dataseries is restricted to specific users '
end

/* is this one of the OWNERS? */
if pos('*',owners)=0 then do            /* * means "any user" */
if wordpos(translate(username),translate(owners))=0 then do
   return 'AUTH  Access to this dataseries is restricted to specific users '
end
end

/* One of the OWNERS, now check password */
stuff=sreh2_get_userinfo(authh,host_nickname,idinfo)
if stuff='' then do
  return 'AUTH   Access to this dataseries is restricted to specific users. Please enter a new username/password.'
end

/*  on comment add, do NOT set cookie
parse var uri thisaddon '?' .
jj=lastpos('/',thisaddon)
pp=left(thisaddon,jj)
foo=sreh2_set_cookie(lookcook,1,'30m',pp,,1) 
*/

return 1

